#! /bin/sh
# (next line not seen by tcl) \
exec tclsh $0 ${1+"$@"}

############################################################################
### mkexpt.tcl: make scenariofiles/scripts for ecc-scaled polar multiple imgs
# --this file contains everything to make/run expt *except* bitmaps in ./tiff
# --makes 4 unique 512 sec runs (2 CCW, 2 CW)
# --eccentricity-scaled moving wedge of pol-jittered images (e.g., faces)
# --image at fairly shuffled ecc chosen for update + new poljitt
# --main adjustable global parms below
# --bitmaps extending beyond winy max/min clamped to stay completely onscreen
# --def needs XGA window (resize: edit pixel parms: {min,max}eccimg,win{x,y})
# --run time for this script: ~1 min for 4 runs
# --v0.1a: first hack (marty sereno, Aug 2009)
# --v0.1b: csh scripts create scan-specific logdirs, use {Src}MapperEnv.csh
# --v0.1c: task monitor ??-180rot.tiff upsidedown face(s), -hkcode so logged
# --v0.1d: def dump sh script
############################################################################

### adjustable parms (first hack at settings)
set simulimgs 4        ;# (4) number simultaneous images
set newimgsoa 15       ;# (15) frames before next image update
set poljitt  15        ;# (15) degrees jitter from pol driver
set polwidth 60.0      ;# (60.0) wedge mask width (nomask=0.0)
set mineccimg 30       ;# (30,20) min eccen image center, pix
set maxeccimg 350      ;# (350,250) max eccen image center, pix
set indir tiffs        ;# subdir
set scriptinfix faces  ;# for scenariofile, runscript names
set targinfix -180rot  ;# filename infix for targets: e.g., zq-180rot.tiff
set winx 1024          ;# (1024,640) same as <x> in -winxy
set winy 768           ;# (768,640) same as <y> in -winxy
set seed 1             ;# num for same rand seq, or "noseed" for new each time
set mkshflag 1         ;# else runscripts are csh

### return int up to but not including limit (simulate sgitcl random)
proc random { limit } {
  return [expr int($limit.0 * rand())]
}

### return avg image size (not yet used)
proc avgsize { list } {
  set x 0.0
  set y 0.0
  set n 0
  foreach file $list {
    set dims [exec identify -format "%w %h" $file]
    set x [expr $x + [lindex $dims 0]]
    set y [expr $y + [lindex $dims 1]]
    incr n
  }
  return "[expr $x/$n] [expr $y/$n]"
}

### return shuffled list
proc shufflelist { list } {
  set i 0
  set n [llength $list]
  set max [expr $n - 1]
  while {$i < $max} {
    set j [expr [random [expr $n - $i]] + $i]
    set tmp [lindex $list $j]
    set list [lreplace $list $j $j [lindex $list $i]]
    set list [lreplace $list $i $i $tmp]
    incr i
  }
  return $list
}

### generate frame-by-frame pol ang eccen scaled images (TODO: task)
proc mktask { stimstemtype } {
  global simulimgs newimgsoa poljitt polwidth mineccimg maxeccimg
  global indir scriptinfix targinfix winx winy mkshflag

  ### images
  if ![file exists $indir] {
    puts "mkexpts.tcl: ### missing image source dir: ./$indir"
    exit
  }
  set imglist [glob -nocomplain $indir/*.tiff]
  if ![llength $imglist] {
    puts "mkexpts.tcl: ### no images found in image source dir: ./$indir"
    exit
  }
  set imgcnt [llength $imglist]
  puts "mkexpt.tcl: making $stimstemtype.{scn,csh} (unique images: $imgcnt)"

  ### constants and standard parms
  set deg2rad [expr 3.14159265/180.0]
  set soaframe 16.6667  ;# 1/72=13.8889, 1/60=16.6667
  set cyc 8             ;# (8) polar phase encode cycle count
  #set cyc 1            ;# testing
  set cyclengthsec 64   ;# (64) polar phase encode cycle length
  set framespersec 60   ;# (60) framerate

  ### init
  set totalframes [expr $cyc * $cyclengthsec * $framespersec]
  set eccrange [expr $maxeccimg - $mineccimg]
  set framenum 0
  if {$stimstemtype == "pol${scriptinfix}1"}  { set cwflag 0 }
  if {$stimstemtype == "pol${scriptinfix}2"}  { set cwflag 1 }
  if {$stimstemtype == "pol${scriptinfix}1b"} { set cwflag 0 }
  if {$stimstemtype == "pol${scriptinfix}2b"} { set cwflag 1 }
  set a [expr pow($maxeccimg/$mineccimg,1.0/($maxeccimg-$mineccimg))]
  set currimgorder {}
  for {set i 0} {$i < $simulimgs} {incr i} { ;# init small array imgs,ecc's
    lappend currimgorder $i
    set currimgnums($i) [random $imgcnt]
    set ecc [expr $mineccimg + $eccrange/($simulimgs-1) * $i]
    set currimgeccs($i) [expr $mineccimg*pow($a,$ecc-$mineccimg)]
    set currimgjitts($i) \
      [expr $deg2rad * ( [random $poljitt].0 - ($poljitt.0/2.0) ) ]

  }
  set currimgorder [shufflelist $currimgorder]

  ### make scenario file
  set targflag 0
  set lasttargflag 0
  set targcnt 0
  set frames2newimg 0   ;# trigger restart countdown
  set j 0
  set scnfile $stimstemtype.scn
  set id [open $scnfile w 0644]
  while {$framenum < $totalframes} {
    if {$frames2newimg == 0} {   ;# rotate in one new image, re-jitter it
      set k [lindex $currimgorder $j]
      set currimgorder [shufflelist $currimgorder]
      set currimgnums($k) [random $imgcnt]
      set currimgjitts($k) \
        [expr $deg2rad * ( [random $poljitt].0 - ($poljitt.0/2.0) ) ]
      #no jitter ecc for now
      set frames2newimg $newimgsoa  ;# restart countdown
      incr j
      if {$j == $simulimgs} { set j 0 }  ;# wrap
    }
    ## periodic driver
    set pol0 [expr 360.0*$deg2rad*$framenum.0/($framespersec*$cyclengthsec)]
    set soa $soaframe
    set targflag 0
    set code -1  ;# non-event: -1,99 not logged
    set i [expr $simulimgs - 1]
    while {$i >= 0} {  ;# set code if this event/frame has 1st frame of targ
      set rgb [lindex $imglist $currimgnums($i)]
      # target freq determined by proportion of targs (curr: ~2/min)
      if [string match $indir/*${targinfix}.tiff $rgb] {
        set targflag 1
        if {!$lasttargflag} {
          set code 0
          puts "mkexpts.tcl: targ:$rgb t=[expr $framenum/$framespersec]s"
          incr targcnt
        }
      }
      incr i -1
    }
    set i [expr $simulimgs - 1]   ;# draw periph imgs first
    while {$i >= 0} {  ;# generate one multi-line frame of $simulimgs imgs
      set rgb [lindex $imglist $currimgnums($i)]
      set ecc $currimgeccs($i)
      set pol [expr $pol0 + $currimgjitts($i)]
      if {$cwflag} { set pol -$pol }
      set xoff [expr round($ecc * (cos($pol)))]
      set yoff [expr round($ecc * (sin($pol)))]
      set size [expr round($ecc)]  
      set maxyoff [expr ($winy - $size)/2]  ;# lblo: yoff is at middle of tiff
      if {$yoff < -$maxyoff } { set yoff -$maxyoff } ;# else bad glRasterPos
      if {$yoff >  $maxyoff } { set yoff  $maxyoff } ;# would draw, for symm
      if {$i == [expr $simulimgs - 1] } {  ;# first
        puts $id \
          "$soa $soa $code rgb=./$rgb xoff=$xoff yoff=$yoff lblo=5 size=$size +"
      } elseif {$i > 0} {
        puts $id "         rgb=./$rgb xoff=$xoff yoff=$yoff lblo=5 size=$size +"
      } else { ;# final
        puts $id "         rgb=./$rgb xoff=$xoff yoff=$yoff lblo=5 size=$size"
      }
      incr i -1
    }
    set lasttargflag $targflag
    incr framenum
    incr frames2newimg -1
  } ;# for every frame
  close $id
  puts "mkexpt.tcl: $targcnt targets in [expr $totalframes/$framespersec]s"
  set rate [expr $targcnt.0/($totalframes.0/$framespersec.0/60.0)]
  puts "mkexpt.tcl: [format "%2.1f" $rate]/min"

  ### generate standalone executable script to setup path/USB/logs, run mapper
  if {$mkshflag} {
    set id [open $stimstemtype.sh w 0755]
    puts $id "#! /bin/sh"
    puts $id ""
    puts $id \
 "trap 'sleep 4; if \[ -e \$scnfile \]; then gzip $scnfile; echo rezip; fi' INT"
    puts $id ""
    puts $id "### setup paths"
    puts $id "if \[ -e ../../../MapperEnv.sh \]; then"
    puts $id "  source ../../../MapperEnv.sh"
    puts $id "else"
    puts $id \
      "  echo \"$stimstemtype.sh: ### ../../../MapperEnv.sh not found\""
    puts $id "  exit"
    puts $id "fi"
    puts $id ""
    puts $id "### defaults (overrideable by setenv in current shell)"
    puts $id "if \[ -z \$PULSECNT \]; then export PULSECNT=0; fi"
    puts $id "if \[ -z \$STIMWINX \]; then export STIMWINX=$winx; fi"
    puts $id "if \[ -z \$STIMWINY \]; then export STIMWINY=$winy; fi"
    puts $id \
      "if \[ -z \$MAPPERPROG \]; then export MAPPERPROG ../../../mapper; fi"
    puts $id ""
    puts $id "### maybe load USB<->TTL box"
    puts $id "echo \"setup USB\? ('yes' or 'no')\""
    puts $id "read resp"
    puts $id "if \[ \"\$resp\" = \"yes\" \] || \[ \"\$resp\" = \"y\" \]; then"
    puts $id "  loadUSB"
    puts $id "fi"
    puts $id ""
    puts $id "### put logfile into date-name-scannumber subdir"
    puts $id "echo \"\""
    puts $id "if \[ \$PULSECNT -eq -1 \]; then"
    puts $id "  echo \"   start expt:  F1 (or Mac: fn-F1)\""
    puts $id "fi"
    puts $id "echo \" restart expt:  F2 (or Mac: fn-F2)\""
    puts $id "echo \"  quit mapper:  Esc\""
    puts $id "if \[ \$PULSECNT -gt 0 \]; then"
    puts $id "  echo \"waiting for pulse: \$PULSECNT\""
    puts $id "fi"
    puts $id "echo \"\""
    puts $id "echo \"Enter subj initials (e.g., MS):\""
    puts $id "read init"
    puts $id "echo \"Enter scannum(s) with no spaces (e.g., scan02,03):\""
    puts $id "read scan"
    puts $id "session=`date +%y%m%d`\$\{init\}-\$\{scan\}"
    puts $id "if \[ ! -e \$session \]; then mkdir \$session; fi"
    puts $id "if \[ ! -e \$session \]; then"
    puts $id \
    "  echo \"$stimstemtype.sh: can't make session for logfile   ...quitting\""
    puts $id "  exit"
    puts $id "fi"
    puts $id \
      "echo \"$stimstemtype.sh: logfile will be saved in session: \$session\""
    puts $id ""
    puts $id "### auto uncompress"
    puts $id "if \[ \! -e $scnfile \] \&\& \[ -e $scnfile.gz \]; then"
    puts $id "  echo \"uncompressing $scnfile\""
    puts $id "  gunzip $scnfile.gz"
    puts $id "fi"
    puts $id ""
    puts $id "### run hkexpt: ecc-scaled images polar angle"
    puts $id "\$MAPPERPROG -stimtype 60 \\"
    puts $id "  -exptfontsize 24 \\"
    if {$cwflag} {
      puts $id "  -polperiod -$cyclengthsec \\"
    } else {
      puts $id "  -polperiod $cyclengthsec \\"
    }
    puts $id "  -polwidth $polwidth \\"
    puts $id "  -hkcode \\"
    puts $id "  -usbtrigbit 0 B 0 \\"
    puts $id "  -logfiledir \$session \\"
    puts $id "  -exptfile $scnfile \\"
    puts $id "  -winxy \$STIMWINX \$STIMWINY \\"
    puts $id "  -waitforpulse \$PULSECNT &"
    puts $id ""
    puts $id "### raise priority"
    puts $id "allowed=`sudo -l -n &`"
    puts $id "if \[ `echo \$allowed | grep -c renicemapper` \]; then"
    puts $id "  sudo renicemapper &"
    puts $id "fi"
    puts $id ""
    puts $id "### auto recompress"
    puts $id "sleep 4"
    puts $id "if \[ -e $scnfile \]; then"
    puts $id "  gzip $scnfile"
    puts $id "  echo \"scenario file: $scnfile recompressed\""
    puts $id "fi"
    close $id

  } else {  ;# csh
    set id [open $stimstemtype.csh w 0755]
    puts $id "#! /bin/csh -f"
    puts $id ""
    puts $id "onintr cleanup"
    puts $id ""
    puts $id "### setup paths"
    puts $id "if (-e ../../../SrcMapperEnv.csh) then"
    puts $id "  source ../../../SrcMapperEnv.csh"
    puts $id "else if (-e ../../../MapperEnv.csh) then"
    puts $id "  source ../../../MapperEnv.csh"
    puts $id "else"
    puts $id \
      "  echo \"$stimstemtype.csh: ### ../../../{Src}MapperEnv.csh not found\""
    puts $id "  exit"
    puts $id "endif"
    puts $id ""
    puts $id "### defaults (overrideable by setenv in current shell)"
    puts $id "if (\! \$\?PULSECNT) setenv PULSECNT 0"
    puts $id "if (\! \$\?STIMWINX) setenv STIMWINX $winx"
    puts $id "if (\! \$\?STIMWINY) setenv STIMWINY $winy"
    puts $id "if (\! \$\?MAPPERPROG) setenv MAPPERPROG ../../../mapper"
    puts $id ""
    puts $id "### maybe load USB<->TTL box"
    puts $id "echo \"setup USB\? ('yes' or 'no')\""
    puts $id "set resp = \"\$\<\""
    puts $id "if (\"\$resp\" == \"yes\" || \"\$resp\" == \"y\") then"
    puts $id "  loadUSB"
    puts $id "endif"
    puts $id ""
    puts $id "### put logfile into date-name-scannumber subdir"
    puts $id "echo \"\""
    puts $id "if (\$PULSECNT == -1) echo \"   start expt:  F1 (or Mac: fn-F1)\""
    puts $id "echo \" restart expt:  F2 (or Mac: fn-F2)\""
    puts $id "echo \"  quit mapper:  Esc\""
    puts $id "if (\$PULSECNT \> 0) echo \"waiting for pulse: \$PULSECNT\""
    puts $id "echo \"\""
    puts $id "echo \"Enter subj initials (e.g., MS):\""
    puts $id "set init = \"\$\<\""
    puts $id "echo \"Enter scannum(s) with no spaces (e.g., scan02,03):\""
    puts $id "set scan = \"\$\<\""
    puts $id "set session = `date +%y%m%d`\$\{init\}-\$\{scan\}"
    puts $id "if (! -e \$session) mkdir \$session"
    puts $id "if (! -e \$session) then"
    puts $id \
    "  echo \"$stimstemtype.csh: can't make session for logfile   ...quitting\""
    puts $id "  exit"
    puts $id "endif"
    puts $id \
      "echo \"$stimstemtype.csh: logfile will be saved in session: \$session\""
    puts $id ""
    puts $id "### auto uncompress"
    puts $id "if (\! -e $scnfile \&\& -e $scnfile.gz) then"
    puts $id "  echo \"uncompressing $scnfile\""
    puts $id "  gunzip $scnfile.gz"
    puts $id "endif"
    puts $id ""
    puts $id "### run hkexpt: ecc-scaled images polar angle"
    puts $id "\$MAPPERPROG -stimtype 60 \\"
    puts $id "  -exptfontsize 24 \\"
    if {$cwflag} {
      puts $id "  -polperiod -$cyclengthsec \\"
    } else {
      puts $id "  -polperiod $cyclengthsec \\"
    }
    puts $id "  -polwidth $polwidth \\"
    puts $id "  -hkcode \\"
    puts $id "  -usbtrigbit 0 B 0 \\"
    puts $id "  -logfiledir \$session \\"
    puts $id "  -exptfile $scnfile \\"
    puts $id "  -winxy \$STIMWINX \$STIMWINY \\"
    puts $id "  -waitforpulse \$PULSECNT &"
    puts $id ""
    puts $id "### raise priority"
    puts $id "set allowed = `sudo -l -n &`"
    puts $id "if (`echo \$allowed | grep -c renicemapper`) sudo renicemapper &"
    puts $id ""
    puts $id "### auto recompress"
    puts $id "cleanup:"
    puts $id "sleep 4"
    puts $id "if (-e $scnfile) then"
    puts $id "  gzip $scnfile"
    puts $id "  echo \"scenario file: $scnfile recompressed\""
    puts $id "endif"
    close $id
  }

  puts "done"
}

### generate files
if { $seed != "noseed" } { expr srand($seed) }
mktask pol${scriptinfix}1
mktask pol${scriptinfix}2
mktask pol${scriptinfix}1b
mktask pol${scriptinfix}2b

### compress
foreach runinfix "1 2 1b 2b" {
  set scnfile pol${scriptinfix}${runinfix}.scn
  if [file exists ${scnfile}.gz] { exec rm ${scnfile}.gz }
  puts "compressing $scnfile"
  exec gzip $scnfile
}
puts "done compressing"

